<?php

namespace TVn7\TresorerieBundle\Entity;

use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Blameable\Traits\BlameableEntity;
use Gedmo\Timestampable\Traits\TimestampableEntity;
use TVn7\UserBundle\Entity\Membre;
use Gedmo\Mapping\Annotation as Gedmo;

/**
 * AccountingYear
 *
 * @ORM\Table(name="tdb_accounting_year")
 * @ORM\Entity(repositoryClass="TVn7\TresorerieBundle\Repository\AccountingYearRepository")
 * @Gedmo\Loggable(logEntryClass="TVn7\LogBundle\Entity\AppsLog")
 */
class AccountingYear
{
    use BlameableEntity;
    use TimestampableEntity;

    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var Membre
     *
     * @ORM\OneToOne(targetEntity="TVn7\UserBundle\Entity\Membre")
     * @ORM\JoinColumn(name="accountant_id", referencedColumnName="id", nullable=false)
     * @Gedmo\Versioned
     */
    private $accountant;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="start_date", type="date")
     * @Gedmo\Versioned
     */
    private $startDate;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="end_date", type="date", nullable=true)
     * @Gedmo\Versioned
     */
    private $endDate;

    /**
     * @var bool
     *
     * @ORM\Column(name="is_editable", type="boolean")
     * @Gedmo\Versioned
     */
    private $isEditable;

    /**
     * @var Collection
     * @ORM\OneToMany(targetEntity="TVn7\TresorerieBundle\Entity\Transaction", mappedBy="accountingYear")
     * @ORM\OrderBy({"date" = "DESC"})
     *
     */
    private $transactions;

    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set accountantId
     *
     * @param Membre $accountant
     *
     * @return AccountingYear
     */
    public function setAccountant(Membre $accountant)
    {
        $this->accountant = $accountant;

        return $this;
    }

    /**
     * Get accountantId
     *
     * @return Membre
     */
    public function getAccountant()
    {
        return $this->accountant;
    }

    /**
     * Set startDate
     *
     * @param \DateTime $startDate
     *
     * @return AccountingYear
     */
    public function setStartDate($startDate)
    {
        $this->startDate = $startDate;

        return $this;
    }

    /**
     * Get startDate
     *
     * @return \DateTime
     */
    public function getStartDate()
    {
        return $this->startDate;
    }

    /**
     * Set endDate
     *
     * @param \DateTime $endDate
     *
     * @return AccountingYear
     */
    public function setEndDate($endDate)
    {
        $this->endDate = $endDate;

        return $this;
    }

    /**
     * Get endDate
     *
     * @return \DateTime
     */
    public function getEndDate()
    {
        return $this->endDate;
    }

    /**
     * Set isEditable
     *
     * @param boolean $isEditable
     *
     * @return AccountingYear
     */
    public function setIsEditable($isEditable)
    {
        $this->isEditable = $isEditable;

        return $this;
    }

    /**
     * Get isEditable
     *
     * @return bool
     */
    public function getIsEditable()
    {
        return $this->isEditable;
    }


    /**
     * @return Collection
     */
    public function getTransactions()
    {
        return $this->transactions;
    }

    /**
     * @param Collection $transactions
     * @return AccountingYear
     */
    public function setTransactions($transactions)
    {
        $this->transactions = $transactions;
        return $this;
    }

    public function getTotalIncome()
    {
        $incomes = $this->transactions->filter(function (Transaction $f) {
            return $f->isIncome();
        });

        $totalIncome = 0.0;
        foreach ($incomes as $income) {
            /** @var Transaction $income */
            $totalIncome += $income->getAmount();
        }

        return $totalIncome;
    }

    public function getTotalExpense()
    {
        $incomes = $this->transactions->filter(function (Transaction $f) {
            return $f->isExpense();
        });

        $totalExpense = 0.0;
        foreach ($incomes as $income) {
            /** @var Transaction $income */
            $totalExpense += $income->getAmount();
        }

        return $totalExpense;
    }

    public function __toString()
    {
        return $this->getAccountant()->getUsername() . ' ' . $this->getStartDate()->format('d/m/Y') . '-' . (empty($this->getEndDate()) ? '' : $this->getEndDate()->format('d/m/Y'));
    }

}

